#include <iostream>
#include <vector>
#include <iterator>

using namespace std;

/*
    Bejárók: a mutatók általánosításai

    Minden adatszerkezetnek van (több) saját bejárója, amelyek beágyazott osztályok lehetnek:
    - iterator: egyszerű bejáró
        o input: másoló konstruktor, értékadás operátor, inkrementálás, egyenlőség vizsgálat, dereferálás jobbértékként
        o output: másoló konstruktor, értékadás operátor, inkrementálás, dereferálás balértékként
        o forward: input + output + alapértelmezett konstruktor
        o bidirectional: forward + dekrementálás
        o random access: bidirectional + a következõ operátorok: +, -, +=, -=, >, <, >=, <=, []
    - reverse_iterator: fordított bejáró
        o kizárólag bidirectional és random access iterátorokhoz
    - const_iterator: konstans bejáró, nem enged értékváltoztatást
    ( - insert_iterator, front_insert_iterator, back_insert_iterator, istream_iterator, ostream_iterator)

    Bejárókra értelmezett példa műveletek:
    - ++it / it++: elõrelépés (fordított bejárónál vissza)
    - --it / it--: visszalépés (legalább kétirányú iterátor kell)
    - it: aktuális elem mutatója
    - *it: aktuális elem
    - it = x.begin(): adatszerkezet eleje egyszerû bejáróval
    - it = x.end(): adatszerkezet vége egyszerû bejáróval
    - it = x.rend(): adatszerkezet eleje fordított bejáróval
    - it = x.rbegin(): adatszerkezet vége fordított bejáróval
    - it = x.cbegin(): adatszerkezet eleje konstans bejáróval (C++11)
    - it = x.cend(): adatszerkezet vége konstans bejáróval (C++11)
*/

int main()
{
    vector<char> v;
    v.push_back('A');
    v.push_back('B');
    v.push_back('C');
    v.push_back('D');
    v.push_back('E');
    v.push_back('F');
    v.push_back('G');

    cout << "Vektor elore:" << endl;
    vector<char>::iterator it;  // Bejáró szerkezet
    for (it = v.begin();        // Vektor elejére állítás
         it != v.end();         // Amíg nem értünk a vektor végére
         ++it)                  // Léptetés elõre
        cout << *it << " ";
    cout << endl;

    cout << "Vektor vissza:" << endl;
    for (vector<char>::reverse_iterator rev_it = v.rbegin();   // Fordított bejáró, végére állítás
         rev_it != v.rend();                                    // Amíg nem értünk az elejére
         ++rev_it)                                              // Léptetés vissza
        cout << *rev_it << " ";
    cout << endl;

    v.insert(it, 'H');          // Beszúrás a bejáró aktuális helye elé

    cout << "Beszuras utan:\n";
    for (it = v.begin(); it != v.end(); ++it)
        cout << *it << " ";
    cout << endl;
    
    cout << "Elemek kettesevel:" << endl;
    for (it = v.begin();            // Vektor elejére állítás
         it != v.end();             // Amíg nem értünk a vektor végére
         advance(it, 2))            // Léptetés kettesével elõre
        cout << *it << " ";
    cout << endl;
    
    cout << "Vektor hossza: " << distance(v.begin(), v.end()) << endl;   // Iterátorok távolsága

    return 0;
}
